%  pi = CTMCSolve(Q)
%  
%  Computes the stationary solution of a continuous time 
%  Markov chain.
%  
%  Parameters
%  ----------
%  Q : matrix, shape (M,M)
%      The generator matrix of the Markov chain
%      
%  Returns
%  -------
%  pi : row vector, shape (1,M)
%      The vector that satisfies `\pi\, Q = 0, \sum_i \pi_i=1`
%  
%  Notes
%  -----
%  The procedure raises an exception if :code:`butools.checkInput` 
%  is set to :code:`true` and :func:`CheckGenerator(Q)` fails.

function pi = CTMCSolve(Q)

    M = Q;
    M(:,1) = 1;
    m = zeros(1,size(M,1));
    m(1) = 1.0;
    pi = m/M;
end

