%  cdf = CdfFromPH(alpha, A, x)
%  
%  Returns the cummulative distribution function of a
%  continuous phase-type distribution.
%  
%  Parameters
%  ----------
%  alpha : matrix, shape (1,M)
%      The initial probability vector of the phase-type
%      distribution.
%  A : matrix, shape (M,M)
%      The transient generator matrix of the phase-type
%      distribution.
%  x : vector of doubles
%      The cdf will be computed at these points
%  
%  Returns
%  -------
%  cdf : column vector of doubles
%      The values of the cdf at the corresponding "x" values

function cdf = CdfFromPH (alpha, A, x)

    cdf = zeros(length(x),1);
    for i=1:length(x)
        cdf(i) = 1.0 - sum(alpha*expm(A*x(i)));
    end
end

