%  moms = MomentsFromPH(alpha, A, K, prec)
%  
%  Returns the first K moments of a continuous phase-type
%  distribution.
%  
%  Parameters
%  ----------
%  alpha : vector, shape (1,M)
%      The initial probability vector of the phase-type
%      distribution.
%  A : matrix, shape (M,M)
%      The transient generator matrix of the phase-type
%      distribution.
%  K : int, optional
%      Number of moments to compute. If K=0, 2*M-1 moments
%      are computed. The default value is K=0.
%  prec : double, optional
%      Numerical precision for checking the input.
%      The default value is 1e-14.
%  
%  Returns
%  -------
%  moms : row vector of doubles
%      The vector of moments.

function moms = MomentsFromPH (alpha, A, K)

    moms = zeros(1,K);
    iA = inv(-A);
    for i=1:K
        moms(i) = factorial(i) * sum(alpha*iA^i);
    end
end
