function example1 ()

% parameters of the queue
B = [6, 1, 0; 0, 4, 1; 2, 0, 0];
F = [2, 1, 0; 5, 0, 0; 1, 3, 0];
L = [-15, 3, 2; 0, -14, 4; 3, 1, -10];

L0 = [-6, 0, 3; 4, -12, 3; 1, 1, -6];

XS = [0.12, 0.01, 0.02; 0, 0.07, 0.04; 0.1, 0.03, 0.08];
YS = [0.5, 0.2, 0.15; 0.89, 0, 0; 0.79, 0, 0];

XA = [0.04, 0.2, 0.03; 0.09, 0, 0.02; 0.07, 0.05, 0.01];
YA = [0.13, 0.25, 0.35; 0.59, 0.2, 0.1; 0.7, 0.17, 0];

N = size(L,1);

% check stability
ph = CTMCSolve(B*inv(eye(N)-XS)*YS+L+F*inv(eye(N)-XA)*YA);
driftup = sum(ph*F*inv(eye(N)-XA)^2*YA)
driftdown = sum(ph*B*inv(eye(N)-XS)^2*YS)

% functions to obtain matrices R, V and H

    function [Rx, Vx, Hx] = getRVHlinear()
        Li = inv(-L);
        Hx = zeros(N);
        Vx = YA*Li;
        Rx = Vx*F+XA;
        oV = Vx + 1; 
        oH = Hx + 1;
        oR = Rx + 1;
        while max([norm(oV-Vx),norm(oH-Hx),norm(oR-Rx)])>1e-14
            oV = Vx; oH = Hx; oR = Rx;
            Hx = dlyap(Rx, XS, Rx*Vx*B);
            Vx = (Hx*YS + YA)*Li;
            Rx = Vx*F+XA;
        end
    end

    function [Rx, Vx, Hx] = getRVHfromQBD()
        Fh = [zeros(N), F, zeros(N); zeros(N), XA, zeros(N); zeros(N,3*N)];
        Lh = [L, zeros(N), zeros(N); YA, -eye(N), zeros(N); YS, zeros(N), -eye(N)];
        Bh = [zeros(N), zeros(N), B; zeros(N,3*N); zeros(N), zeros(N), XS];
        [~,Rh] = QBD_CR(Bh, Lh, Fh, 'MaxNumIt', 50);
        Uh = Lh+Rh*Bh;
        M = [zeros(N),-eye(N),zeros(N)]*inv(Uh);
        Vx = M(:,1:N);
        Hx = M(:,2*N+1:end);
        Rx = XA + Vx*F;
    end

    function [Rx, Vx, Hx] = getRVHfromNI()
        Li = inv(-L);
        Zs1 = YS*Li*F;
        Za1 = YA*Li*B;
        Zs2 = XS + YS*Li*B;
        Za2 = XA + YA*Li*F;
        Hx = zeros(N);
        oH = Hx + 1;
        while norm(oH-Hx)>1e-14
            oH = Hx;
            T1 = Hx*Zs1+Za2;
            T2 = Hx*Zs2+Za1;
            iT2 = inv(eye(N)-Zs1*T2);
            D = dlyap(T1, Zs2*iT2, T1*T2*iT2-Hx*iT2);
            Hx = Hx + D;
        end
        Vx = Hx*YS*Li + YA*Li;
        Rx = XA + Vx*F;
    end

% speed comparison
tic
[R, V, H] = getRVHlinear();
toc

tic
[R, V, H] = getRVHfromNI();
toc

tic
[R, V, H] = getRVHfromQBD();
toc

% obtain initial values x1, v0, y0
M = XS+YS*inv(-L0)*F*(V*B+H*XS)-eye(N);
M(:,1) = sum(YS*inv(-L0)+YS*inv(-L0)*F*inv(eye(N)-R)*V,2);
y0 = [1, zeros(1,N-1)] * inv(M);
v0 = y0*YS*inv(-L0);
x1 = v0*F;

lambda = sum(v0*F*inv(eye(N)-R));
mu = sum(v0*F*(V*B+H*XS)*inv(eye(N)-XS) + v0*F*inv(eye(N)-R)*H);

x1 = x1/lambda;
y0 = y0/mu;


% Queue length distribution: vi = v0 * F * R^(i-1) * V
numOfQLProbs = 50;
qlProbs = zeros(1,numOfQLProbs);
qlProbs(1) = sum(v0);
for m=1:numOfQLProbs-1
    qlProbs(m+1) = sum(v0*F*R^(m-1)*V);
end
qlProbs = qlProbs'

% Queue length moments
numOfQLMoms = 5;
qlMoms = zeros(1,numOfQLMoms);
iR = inv(eye(N)-R);
for m=1:numOfQLMoms
    qlMoms(m) = factorial(m)*sum(v0*F*iR^(m+1)*R^(m-1)*V);
end
qlMoms = MomsFromFactorialMoms(qlMoms)

% DPH representation for the queue length distribution:
xi = v0*F*inv(eye(N)-R);
Delta = diag(xi);
tau = ones(1,N)*V'*Delta;
T = pinv(Delta)*R'*Delta;

qlProbsFromDPH = PmfFromDPH(tau,T,0:numOfQLProbs-1)
qlMomsFromDPH = MomentsFromDPH(tau,T,numOfQLMoms)

% Sojourn time distribution
points=0:0.1:10;
ini = kron(ones(1,N),x1*inv(eye(N)-R));
Z = kron((L+F*inv(eye(N)-XA)*YA)',eye(N)) + kron(YS',eye(N))*inv(eye(N*N)-kron(XS',R))*kron(B',R);
clo = reshape(inv(eye(N)-XA)*YA, N^2, 1);
cdf=zeros(length(points),1);
for i=1:length(points)
    cdf(i) = 1-ini*expm(Z*points(i))*clo;
end
stCdf = cdf

% Sojourn time moments
numOfSTMoms = 5;
stMoms = zeros(1,numOfSTMoms);
iZ = inv(-Z);
for m=1:numOfSTMoms
    stMoms(m) = factorial(m)*sum(ini*(-Z)*iZ^(m+1)*clo);
end
stMoms

% PH representation of the sojourn time distribution:
Delta = diag(x1*inv(eye(N)-R));
kappa = reshape(inv(eye(N)-XA)*YA, N^2, 1)' * kron(eye(N),Delta);
K = kron(L+F*inv(eye(N)-XA)*YA,eye(N)) + kron(B,pinv(Delta)*R'*Delta)*inv(eye(N*N)-kron(XS,pinv(Delta)*R'*Delta))*kron(YS,eye(N));

stCdfFromPH = CdfFromPH(kappa,K,points)
stMomsFromPH = MomentsFromPH(kappa,K,numOfSTMoms)



end