function example2 ()

    function [Rx, Vx, Hx] = getRVHlinear(L, F, XA, YA, B, XS, YS)
        N = size(L,1);
        Li = inv(-L);
        Hx = zeros(N);
        Vx = YA*Li;
        Rx = Vx*F+XA;
        oV = Vx + 1; 
        oH = Hx + 1;
        oR = Rx + 1;
        while max([norm(oV-Vx),norm(oH-Hx),norm(oR-Rx)])>1e-13*N
            oV = Vx; oH = Hx; oR = Rx;
            Hx = dlyap(Rx, XS, Rx*Vx*B);
            Vx = (Hx*YS + YA)*Li;
            Rx = Vx*F+XA;
        end
    end

    function [Rx, Vx, Hx] = getRVHfromQBD(L, F, XA, YA, B, XS, YS)
        N = size(L,1);
        Fh = [zeros(N), F, zeros(N); zeros(N), XA, zeros(N); zeros(N,3*N)];
        Lh = [L, zeros(N), zeros(N); YA, -eye(N), zeros(N); YS, zeros(N), -eye(N)];
        Bh = [zeros(N), zeros(N), B; zeros(N,3*N); zeros(N), zeros(N), XS];
        [~,Rh] = QBD_CR(Bh, Lh, Fh, 'MaxNumIt', 50);
        Uh = Lh+Rh*Bh;
        M = [zeros(N),-eye(N),zeros(N)]*inv(Uh);
        Vx = M(:,1:N);
        Hx = M(:,2*N+1:end);
        Rx = XA + Vx*F;
    end

    function [Rx, Vx, Hx] = getRVHfromNI(L, F, XA, YA, B, XS, YS)
        N = size(L,1);
        Li = inv(-L);
        Zs1 = YS*Li*F;
        Za1 = YA*Li*B;
        Zs2 = XS + YS*Li*B;
        Za2 = XA + YA*Li*F;
        Hx = zeros(N);
        oH = Hx + 1;        
        while norm(oH-Hx)>1e-13*N
            oH = Hx;
            T1 = Hx*Zs1+Za2;
            T2 = Hx*Zs2+Za1;
            iT2 = inv(eye(N)-Zs1*T2);
            D = dlyap(T1, Zs2*iT2, T1*T2*iT2-Hx*iT2);
            Hx = Hx + D;
        end
        Vx = Hx*YS*Li + YA*Li;
        Rx = XA + Vx*F;
    end

    function [Q0, Q1, bX, bY] = buildOnOff (meanOn, meanOff, batchX, drift, K)
        
        % generator of the background process
        Q = zeros(K);
        for i=1:K
            if i<K
                Q(i,i+1) = (K-i) / meanOn;
            end
            if i>1
                Q(i,i-1) = (i-1) / meanOff;
            end
        end
        Q = Q - diag(sum(Q,2));
        theta = CTMCSolve(Q);
        % adjust "rate" parameter such that the target drift is reached
        c = drift / sum(theta*diag(0:K-1)/(1-batchX));
        Q1 = c*diag(0:K-1);
        Q0 = Q - Q1;
        bX = batchX * eye(K);
        bY = eye(K)-bX;
    end

    function [v0, F, R, V] = steadystate(D0, D1, XAd, YAd, S0, S1, XSs, YSs, method)
        NA = size(D0,1);
        NS = size(S0,1);
        B = kron(eye(NA),S1);
        F = kron(D1,eye(NS));
        L = kron(eye(NA),S0) + kron(D0,eye(NS));
        L0 = kron(D0,eye(NS));
        XA = kron(XAd,eye(NS));
        YA = kron(YAd,eye(NS));
        XS = kron(eye(NA),XSs);
        YS = kron(eye(NA),YSs);
        N = size(L,1);
        
        if strcmp(method,'rfi')
            [R, V, H] = getRVHlinear(L, F, XA, YA, B, XS, YS);
        elseif strcmp(method,'rni')
            [R, V, H] = getRVHfromNI(L, F, XA, YA, B, XS, YS);
        elseif strcmp(method,'rqbd')
            [R, V, H] = getRVHfromQBD(L, F, XA, YA, B, XS, YS);
        end

        M = XS+YS*inv(-L0)*F*(V*B+H*XS)-eye(N);
        M(:,1) = sum(YS*inv(-L0)+YS*inv(-L0)*F*inv(eye(N)-R)*V,2);
        y0 = [1, zeros(1,N-1)] * inv(M);
        v0 = y0*YS*inv(-L0);        
        % steady state solution: v0*F*R^{i-1}*V
    end

    function [ini, Z, clo] = sojourntimedep(D0, D1, XAd, YAd, S0, S1, XSs, YSs, method)
        NA = size(D0,1);
        NS = size(S0,1);
        B = kron(eye(NA),S1);
        F = kron(D1,eye(NS));
        L = kron(eye(NA),S0) + kron(D0,eye(NS));
        L0 = kron(D0,eye(NS));
        XA = kron(XAd,eye(NS));
        YA = kron(YAd,eye(NS));
        XS = kron(eye(NA),XSs);
        YS = kron(eye(NA),YSs);
        N = size(L,1);
        
        if strcmp(method,'rfi')
            [R, V, H] = getRVHlinear(L, F, XA, YA, B, XS, YS);
        elseif strcmp(method,'rni')
            [R, V, H] = getRVHfromNI(L, F, XA, YA, B, XS, YS);
        elseif strcmp(method,'rqbd')
            [R, V, H] = getRVHfromQBD(L, F, XA, YA, B, XS, YS);
        end

        M = XS+YS*inv(-L0)*F*(V*B+H*XS)-eye(N);
        M(:,1) = sum(YS*inv(-L0)+YS*inv(-L0)*F*inv(eye(N)-R)*V,2);
        y0 = [1, zeros(1,N-1)] * inv(M);
        v0 = y0*YS*inv(-L0);        
        x1 = v0*F;       
        lambda = sum(x1*inv(eye(N)-R));
        ini = kron(ones(1,N),x1*inv(eye(N)-R)/lambda);
        Z = kron((L+F*inv(eye(N)-XA)*YA)',eye(N)) + kron(YS',eye(N))*inv(eye(N*N)-kron(XS',R))*kron(B',R);
        clo = reshape(inv(eye(N)-XA)*YA, N^2, 1);

        % sojourn time:
        % cdf = 1-ini*expm(Z*t)*clo;
    end


    function [pi0, T, clo] = sojourntime(D0, D1, XAd, YAd, S0, S1, XSs, YSs, Tmethod)
        
        NA = size(D0,1);
        NS = size(S0,1);
        B = kron(eye(NA),S1);
        F = kron(D1,eye(NS));
        L = kron(eye(NA),S0) + kron(D0,eye(NS));
        L0 = kron(D0,eye(NS));
        XA = kron(XAd,eye(NS));
        YA = kron(YAd,eye(NS));
        XS = kron(eye(NA),XSs);
        YS = kron(eye(NA),YSs);
        N = size(L,1);
        
        iXAXS = inv(eye(N)-kron(XAd,XSs));        
        S0v = kron(eye(NA),S0)+kron(eye(NA),S1)*iXAXS*kron(XAd,YSs);
        S1v = kron(eye(NA),S1)*iXAXS*kron(YAd,eye(NS));
        D0v = kron(D0,eye(NS))+kron(D1,eye(NS))*iXAXS*kron(YAd,XSs);
        D1v = kron(D1,eye(NS))*iXAXS*kron(eye(NA),YSs);

        % PROCEDURE 1: Functional iteration
        if strcmp(Tmethod,'tfi')
            T = S0v;
            oT = T+1;
            while norm(T-oT)>1e-13*N
                oT = T;
                X = lyap(T, D0v, S1v);
                T = S0v + X*D1v;
            end      
        else  
            % PROCEDURE 2: Based on R
            if strcmp(Tmethod,'rfi')
                [R, V, H] = getRVHlinear(L, F, XA, YA, B, XS, YS);
            elseif strcmp(Tmethod,'rni')
                [R, V, H] = getRVHfromNI(L, F, XA, YA, B, XS, YS);
            elseif strcmp(Tmethod,'rqbd')
                [R, V, H] = getRVHfromQBD(L, F, XA, YA, B, XS, YS);
            end
            T = kron(eye(NA),S0)+dlyap(R, XS, R*B)*YS;      
            X = lyap(T, D0v, S1v);
        end
        
        if max(abs(1-sum(X*kron(inv(-D0)*D1,inv(eye(NS)-XSs)*YSs),2)))>1e-10
            max(abs(1-sum(X*kron(inv(-D0)*D1,inv(eye(NS)-XSs)*YSs),2)))
        end
        pi0 = DTMCSolve(X*kron(inv(-D0)*D1,inv(eye(NS)-XSs)*YSs));
        pi0 = pi0/sum(pi0*inv(-T));
        clo = sum(kron(eye(NA),S1)*iXAXS+X*kron(D1,eye(NS))*iXAXS*kron(eye(NA),XSs),2);
        nrm = pi0*inv(-T)*clo;
        clo = clo/nrm;
        % everything ready to obtain the sojourn time distribution
        % stCdf = 1-pi0*inv(-T)*expm(T*t)*clo;
        
    end

    % study 1: execution speed of the steady state analysis vs. drift  
    methods = {'rfi','rni','rqbd'};
    exTimesSSVSDrift = zeros(49,4);
    Rep = 30;
    Siz = 5;
    ix = 1;
    for Drift=0.1:0.1:4.9
        
        [D0, D1, XAd, YAd] = buildOnOff (2, 1, 0.35, Drift, Siz);
        [S0, S1, XSs, YSs] = buildOnOff (3, 5, 0.55, 5, Siz);
        
        exTimesSSVSDrift(ix,1) = Drift;
        for m=1:length(methods)
            tim = zeros(1,Rep);
            for i=1:Rep
                tic
                [pi0, T, clo] = steadystate(D0, D1, XAd, YAd, S0, S1, XSs, YSs, methods{m});
                tim(i) = toc;
            end
            exTimesSSVSDrift(ix,m+1) = min(tim);
        end
        exTimesSSVSDrift(ix,:)
        ix = ix + 1;
    end  
    exTimesSSVSDrift
    
    % study 2: execution speed of the steady state analysis vs. input size  
    exTimesSSVSSize = zeros(19,4);
    Drift = 3;
    ix = 1;
    for Siz=2:20
        
        [D0, D1, XAd, YAd] = buildOnOff (2, 1, 0.35, Drift, Siz);
        [S0, S1, XSs, YSs] = buildOnOff (3, 5, 0.55, 5, Siz);
        
        exTimesSSVSSize(ix,1) = Siz;
        for m=1:length(methods)
            tim = zeros(1,Rep);
            for i=1:Rep
                tic
                [pi0, T, clo] = steadystate(D0, D1, XAd, YAd, S0, S1, XSs, YSs, methods{m});
                tim(i) = toc;
            end
            exTimesSSVSSize(ix,m+1) = min(tim);
        end
        exTimesSSVSSize(ix,:)
        ix = ix + 1;
    end  
    exTimesSSVSSize
    
    % study 3: execution speed of the sojourn time analysis vs. input size  
    exTimesSTVSSize = zeros(19,3);
    methods = {'tfi','rni'};
    Rep = 30;
    Drift = 3;
    ix = 1;
    for Siz=2:20
        
        [D0, D1, XAd, YAd] = buildOnOff (2, 1, 0.35, Drift, Siz);
        [S0, S1, XSs, YSs] = buildOnOff (3, 5, 0.55, 5, Siz);
        
        exTimesSTVSSize(ix,1) = Siz;
        for m=1:length(methods)
            tim = zeros(1,Rep);
            for i=1:Rep
                tic
                [pi0, T, clo] = sojourntime(D0, D1, XAd, YAd, S0, S1, XSs, YSs, methods{m});
                tim(i) = toc;
            end
            exTimesSTVSSize(ix,m+1) = min(tim);
        end
        exTimesSTVSSize(ix,:)
        ix = ix + 1;
    end  
    exTimesSTVSSize
    
    % study 4: execution speed of the sojourn time analysis vs. input size  
    exTimesSTVSSize2 = zeros(19,2);
    methods = {'rni'};
    Rep = 30;
    Drift = 3;
    ix = 1;
    for Siz=2:10
        
        [D0, D1, XAd, YAd] = buildOnOff (2, 1, 0.35, Drift, Siz);
        [S0, S1, XSs, YSs] = buildOnOff (3, 5, 0.55, 5, Siz);
        
        exTimesSTVSSize2(ix,1) = Siz;
        for m=1:length(methods)
            tim = zeros(1,Rep);
            for i=1:Rep
                tic
                [ini, Z, clo] = sojourntimedep(D0, D1, XAd, YAd, S0, S1, XSs, YSs, methods{m});
                tim(i) = toc;
            end
            exTimesSTVSSize2(ix,m+1) = min(tim);
        end
        exTimesSTVSSize2(ix,:)
        ix = ix + 1;
    end  
    exTimesSTVSSize2
    
end

